import { useEffect, useMemo, useState } from 'react'
import { BasicTarget, getTargetElement } from '../utils'
import { useSize } from './use-size'

export function useGrid(options: {
  target: BasicTarget
  itemMinWidth?: number
  itemMinHeight?: number
  gutter?: number | [number, number]
}) {
  const [grid, setGrid] = useState<{ rows: number; cols: number }>(() => ({ rows: 0, cols: 0 }))
  const { target, itemMinWidth, itemMinHeight, gutter = 0 } = options

  const [gutterX, gutterY] = useMemo(() => {
    if (typeof gutter === 'number') {
      return [gutter, gutter]
    }
    return gutter
  }, [gutter])

  const size = useSize(target)

  useEffect(() => {
    const el = getTargetElement(target)
    if (el && size) {
      const { paddingTop, paddingBottom, paddingLeft, paddingRight } = getComputedStyle(el)
      const aw = size.width - (parseFloat(paddingLeft) + parseFloat(paddingRight))
      const ah = size.height - (parseFloat(paddingTop) + parseFloat(paddingBottom))

      let [rows, cols] = [1, 1]
      if (typeof itemMinHeight === 'number') {
        rows = Math.floor((ah + gutterY) / (itemMinHeight + gutterY))
      }
      if (typeof itemMinWidth === 'number') {
        cols = Math.floor((aw + gutterX) / (itemMinWidth + gutterX))
      }
      setGrid({ rows, cols })
    }
  }, [gutterX, gutterY, itemMinHeight, itemMinWidth, size, target])

  return grid
}
