import { useEffect, useState, useRef } from 'react'

export const useMouseDown = (target?: HTMLDivElement) => {
  const tar = useRef<Document | HTMLDivElement>(target ? target : document)
  const [state, setState] = useState<boolean>(false)

  const onMouseDown = () => {
    setState(true)
  }

  const onMouseUp = () => {
    setState(false)
  }

  const register = () => {
    tar.current.addEventListener('mousedown', onMouseDown, true)
    tar.current.addEventListener('mouseup', onMouseUp, true)
  }

  const unregister = () => {
    tar.current.removeEventListener('mousedown', onMouseDown, true)
    tar.current.removeEventListener('mouseup', onMouseUp, true)
  }

  useEffect(() => {
    register()
    return unregister
  }, [])

  return state
}
