import { useEffect, useState, useCallback } from 'react'

const DEFAULT_OPTIONS = {
  config: { attributes: true, childList: true, subtree: true },
}
const useMutationObservable = (targetEl: HTMLDivElement, cb: MutationCallback, options = DEFAULT_OPTIONS) => {
  const [observer, setObserver] = useState<MutationObserver>()

  useEffect(() => {
    const obs = new MutationObserver(cb)
    setObserver(obs)
  }, [cb, options, setObserver])

  useEffect(() => {
    if (!observer || !targetEl) return
    const { config } = options
    observer.observe(targetEl, config)
    return () => {
      if (observer) {
        observer.disconnect()
      }
    }
  }, [observer, targetEl, options])
}

/**
 * 元素可见状态监听，用于样式变化、class变化
 * @param target 监听对象
 * @param defVisible 初始化时是否显示，默认为显示状态
 * @returns
 */
export const useMutationVisible = (target?: HTMLDivElement | null, defVisible?: boolean) => {
  const [visible, setVisible] = useState<boolean>(defVisible !== undefined ? defVisible : true)

  const onListMutation = useCallback(
    (mutationList: MutationRecord[]) => {
      for (const mutation of mutationList) {
        if (
          (mutation.type === 'attributes' && mutation.attributeName === 'class') ||
          mutation.attributeName === 'style'
        ) {
          if (target) {
            setVisible(target.offsetWidth > 0 && target.offsetHeight > 0)
          }
        }
      }
    },
    [target, visible, setVisible],
  )

  useMutationObservable(target!, onListMutation)

  return visible
}
