import { DependencyList, RefObject, useEffect, useMemo, useRef } from 'react'
import { useStableCallback } from './use-stable-callback'
import { debounce } from 'lodash'

export function useResize(
  {
    el,
    ref,
    onResize = () => {},
  }: {
    /** @deprecated 建议使用 ref */
    el?: HTMLElement | null
    ref?: RefObject<HTMLElement | null>
    onResize?: (entry: ResizeObserverEntry) => void
  },
  deps: DependencyList = [],
) {
  const prevRect = useRef<ResizeObserverEntry['contentRect']>()
  const onResizeFn = useStableCallback(onResize)

  const obs = useMemo(() => {
    const fn: ResizeObserverCallback = (entries) => {
      entries.forEach((entry) => {
        if (
          prevRect.current?.width !== entry.contentRect.width ||
          prevRect.current?.height !== entry.contentRect.height
        ) {
          onResizeFn?.(entry)
        }
        prevRect.current = entry.contentRect
      })
    }
    return new ResizeObserver(debounce(fn, 5))
  }, [onResizeFn])

  useEffect(() => {
    const ele = ref ? ref.current : el
    if (ele) {
      // 开始监听
      obs?.observe(ele)
      return () => {
        obs?.unobserve(ele)
        obs?.disconnect()
      }
    }
    return undefined
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [obs, el, ref, ...deps])
}
