import { useLayoutEffect, useState } from 'react'
import { BasicTarget, getTargetElement } from '../utils'

interface Size {
  width: number
  height: number
}

export function useSize(target: BasicTarget) {
  const [size, setSize] = useState<Size | undefined>(() => {
    const el = getTargetElement(target)
    return el ? { width: el.clientWidth, height: el.clientHeight } : undefined
  })

  useLayoutEffect(() => {
    const el = getTargetElement(target)
    if (el) {
      const obs = new ResizeObserver((entries) => {
        entries.forEach((entry) => {
          const { clientWidth, clientHeight } = entry.target
          setSize({ width: clientWidth, height: clientHeight })
        })
      })
      obs.observe(el)
      return () => {
        obs.unobserve(el)
        obs.disconnect()
      }
    }
  }, [target])

  return size
}
