import React, { Dispatch, SetStateAction, useImperativeHandle, useMemo, useState } from 'react'
import { Context, createContext } from 'react'

type TSetState<T> = Dispatch<SetStateAction<T>>

enum ETheme {
  dark = 'dark',
  light = 'light',
}

export type TEheme<T> = ETheme & string & T

export interface ICtx<T = any, Config extends Record<string, any> = any> {
  theme: TEheme<T>
  setTheme: TSetState<TEheme<T> | T>
  config: Config
  setConfig: TSetState<Config>
}

export const ThemeContext: Context<ICtx> = createContext<ICtx>({ config: {} } as any)

export interface IThemeRef<T = any, Config extends Record<string, any> = {}> {
  setTheme: (theme: TEheme<T>) => void
  setConfig: (config: Config) => void
}

export const Theme = React.forwardRef<IThemeRef, any>((props, ref) => {
  const { children } = props
  const [theme, setTheme] = useState<ICtx>()
  const [config, setConfig] = useState<Record<string, any>>({})

  useImperativeHandle(ref, () => {
    return {
      setTheme,
      setConfig,
    }
  })

  const memorizedThemeForm = useMemo(() => ({ theme, setTheme, config, setConfig }), [theme, config])

  return <ThemeContext.Provider value={memorizedThemeForm}>{children}</ThemeContext.Provider>
})
